package jp.co.rakus;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.net.ssl.HttpsURLConnection;

import com.fasterxml.jackson.databind.ObjectMapper;

/**
 * APIのサンプル実装
 */
public class SampleGetApi {

    private static final String CRLF = "\r\n";

    // 接続先URLに利用する設定
    // https://●●●●●/■■■■■/api/v1/▲▲▲▲▲/get
    private final static String DOMAIN = "●●●●●";
    private final static String ACCOUNT = "■■■■■";
    private final static String API_KIND = "▲▲▲▲▲";
    private final static String API_URL =
            String.format("https://%s/%s/api/v1/%s/get", DOMAIN, ACCOUNT, API_KIND);

    // APIトークン
    // 各種設定 > 各種設定メニュー > 基本設定 > APIトークン で発行<<APIトークン>>
    private final static String API_TOKEN = "<<APIトークン>>";


    /**
     * メインメソッド
     *
     * @param args コマンドライン引数（未使用）
     * @throws IOException IOエラーが発生した場合
     */
    public static void main(final String[] args) throws IOException {
        final URL url = new URL(API_URL);
        final HttpsURLConnection https = (HttpsURLConnection) url.openConnection();
        https.setRequestMethod("POST");
        https.setRequestProperty("Content-Type", "application/json; charset=utf-8");
        https.setRequestProperty("X-HM-apitoken", API_TOKEN);

        // リクエストボディ作成
        https.setDoOutput(true);
        final StringBuilder requestBody =new StringBuilder();
        final Hashtable<String, Object> requestParams = new Hashtable<String, Object>();
        requestParams.put("search", new HashMap<String, String>() {{
            put("type", "cancel");
            // 日付が必要なAPIの場合
            // put("date", "2020-01-01");
        }});

        final ObjectMapper mapper = new ObjectMapper();
        requestBody.append(mapper.writeValueAsString(requestParams)).append(CRLF);
        final BufferedOutputStream bos = new BufferedOutputStream(https.getOutputStream());
        bos.write(requestBody.toString().getBytes("UTF-8"));
        bos.flush();
        bos.close();

        // API実行
        if (https.getResponseCode() == HttpURLConnection.HTTP_OK) {
            handleSuccessfulResponse(https);
            System.exit(0);
        } else {
            handleFailedResponse(https);
            System.exit(1);
        }
    }

    /**
     * 成功（ステータスコード = 200）時の処理をする。
     *
     * @param https HTTPSコネクション
     * @throws IOException IOエラーが発生した場合
     */
    private static void handleSuccessfulResponse(final HttpsURLConnection https) throws IOException {
        final String fileName = https.getHeaderField("Content-Disposition").replace("attachment; filename*=UTF-8''", "");
        try (BufferedInputStream stream = new BufferedInputStream(https.getInputStream());

                FileOutputStream output = new FileOutputStream(fileName)) {
            final byte[] buf = new byte[256];
            int len;
            while ((len = stream.read(buf)) != -1) {
                output.write(buf, 0, len);
            }
        }
        System.out.println("処理に成功しました");
        System.out.println("  " + fileName);
    }

    /**
     * 失敗（ステータスコード != 200）時の処理をする。
     *
     * @param https HTTPSコネクション
     * @throws IOException IOエラーが発生した場合
     */
    private static void handleFailedResponse(final HttpsURLConnection https) throws IOException {
        final StringBuilder responseBody = new StringBuilder();
        try (BufferedReader reader = new BufferedReader(new InputStreamReader(https.getErrorStream()))) {
            String line;
            while ((line = reader.readLine()) != null) {
                responseBody.append(line + "\n");
            }
        }
        final Map<String, Object> errorJson = parseJson(responseBody.toString());
        System.err.println("処理に失敗しました");

        for (final String key : errorJson.keySet()) {
            System.err.println("  " + key + " : " + errorJson.get(key));
        }
    }

    /**
     * JSONレスポンスを解析する。
     *
     * @param responseBody レスポンス文字列
     * @return レスポンス文字列をJSONとしてパースしたMapオブジェクト
     * @throws IOException IOエラーが発生した場合
     */
    @SuppressWarnings("unchecked")
    private static Map<String, Object> parseJson(final String responseBody) throws IOException {
        // JacksonやJSONICなど任意のJSONライブラリで読み込む。以下はJacksonを使った例
        final ObjectMapper mapper = new ObjectMapper();
        return mapper.readValue(responseBody, Map.class);
    }
}