<?php
/**
 * APIのサンプル
 */

// レスポンスが文字化けする場合は、ご利用の環境にあわせて文字エンコーディングを変換してください。
const FROM_ENCODE = 'UTF-8';
const TO_ENCODE = 'UTF-8';

// 可変
// 接続先URLに利用する設定
// https://●●●●●/■■■■■/api/v1/▲▲▲▲▲/get
$domain = "●●●●●";     // URLのドメイン部分を指定（xxx.hm-f.jpなど）
$accountUrl = "■■■■■"; // アカウント文字列
$apiType = "▲▲▲▲▲";    // APIの種類（例：cnlban）

// APIトークン
// 各種設定 > 各種設定メニュー > 基本設定 > APIトークン で発行<<APIトークン>>
$apiToken = "<<APIトークン>>";

// API用のパラメータを指定
$requestParameters = [
  'search' => [
    'type' => 'cancel'
  ]
];

$requestArray[] = "POST /" . $accountUrl . "/api/v1/" . $apiType . "/get" . " HTTP/1.1";
$requestArray[] = "Host: " . $domain;
$requestArray[] = "Content-Type: application/json";
$requestArray[] = "Content-Length: " . strlen(json_encode($requestParameters));
$requestArray[] = "X-HM-apitoken: " . $apiToken;
$requestArray[] = "X-Content-Type-Options: nosniff";
$requestArray[] = "Cache-Control: no-cache";
$requestArray[] = "Connection: Close";
$requestArray[] = "";
$requestArray[] = json_encode($requestParameters);

$context = stream_context_create();
stream_context_set_option($context, 'ssl', 'verify_peer', false);
stream_context_set_option($context, 'ssl', 'verify_host', false);

$fp = stream_socket_client('ssl://' . $domain . ':443', $errno, $errstr, 10, STREAM_CLIENT_CONNECT, $context);
if (!$fp) {
  die("接続に失敗しました。errorNo : [ " . $errno . " ]  errorMessage : [ " . $errstr . " ]\n");
}
fputs($fp, implode("\r\n", $requestArray));

// レスポンスヘッダ
$responseContentType = "";
$responseCsvFileName = "";
while (!feof($fp)) {
  $temporaryData = fgets($fp, 4096);
  if (preg_match("/^content-type:.+$/i", $temporaryData, $matches)) {
    $responseContentType = $matches[0];
  }
  if (preg_match("/^Content-Disposition:.+$/i", $temporaryData, $matches)) {
    $responseCsvFileName = preg_replace("/^Content-Disposition: attachment; filename\*=UTF-8''/", "", $temporaryData);
  }
  if (preg_match('/^(\r|\n)+$/x', $temporaryData)) {
    break;
  }
}

// 結果出力
$result = "";
if (preg_match("/application\/json/i", $responseContentType)) {
  // json形式
  print '処理に失敗しました' . "\n";
  $responseBody = ""; // レスポンスデータ
  while (!feof($fp)) {
    $responseBody .= fgets($fp, 4096);
  }
  $jsonData = json_decode($responseBody, true);
  $result = outputJson($jsonData);
} else if (preg_match("/text\/csv/i", $responseContentType)) {
  // csv出力
  print '処理に成功しました' . "\n";
  $result = str_repeat(" ", 2) . $responseCsvFileName;
  $filePath = getcwd() . "/" . trim($responseCsvFileName);
  $csvfp = fopen($filePath, "w");
  while (!feof($fp)) {
    $data = fgets($fp, 4096);
    fwrite($csvfp, mb_convert_encoding($data, TO_ENCODE, FROM_ENCODE));
  }
  fflush($csvfp);
  fclose($csvfp);
}
fclose($fp);
print mb_convert_encoding($result, TO_ENCODE, FROM_ENCODE) . "\n";
ob_flush();

/**
 * @param array $jsonData
 * @param int   $depth
 *
 * @return string
 */
function outputJson($jsonData, $depth = 2) {
  $result = "";
  foreach ($jsonData as $key => $value) {
    if (is_array($value)) {
      $result .= str_repeat(" ", $depth) . $key . " :\n";
      $depth += 2;
      $result .= outputJson($value, $depth);
      $depth -= 2;
    } else {
      $result .= str_repeat(" ", $depth) . $key . " : " . $value . "\n";
    }
  }
  return $result;
}
